/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include <time.h>
#include "common.h"
#include "oper_nucomp.h"

mpz_t iqc_disc;
gmp_randstate_t iqc_randst;
static mpz_t valid_c, valid_4a;

//Constructor
IQC_CONSTRUCTOR(common) { 
  mpz_init(iqc_disc);   mpz_init(valid_c);   mpz_init(valid_4a);
  gmp_randinit_default(iqc_randst); gmp_randseed_ui(iqc_randst, time(0)); 
}

//Methods
void iqc_set_disc(mpz_t d) {
  if (mpz_cmp(iqc_disc, d)==0) return;
  mpz_set(iqc_disc, d);
  iqc_nucomp_init();
}

void iqc_get_disc(mpz_t d) {
  mpz_set(d, iqc_disc);
}

int iqc_valid_form(iqc_t a) {
  mpz_neg(valid_c, iqc_disc);
  mpz_addmul(valid_c, a->b, a->b);
  mpz_mul_2exp(valid_4a, a->a, 2);
  return mpz_divisible_p(valid_c, valid_4a);
}
