/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#ifndef LIBIQC_COMMON_MILAN_STRAKA_2008
#define LIBIQC_COMMON_MILAN_STRAKA_2008

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gmp.h>
#ifdef DEBUG
#include <assert.h>
#endif

typedef struct iqc_struct {
  mpz_t a,b;
} iqc_t[1];

extern mpz_t iqc_disc;
extern gmp_randstate_t iqc_randst;
void iqc_set_disc(mpz_t d);
void iqc_get_disc(mpz_t d);
int iqc_valid_form(iqc_t a);

#define iqc_init(x)   do { mpz_init(x->a);      mpz_init(x->b);     } while (0)
#define iqc_clear(x)  do { mpz_clear(x->a);     mpz_clear(x->b);    } while (0)
#define iqc_set(x,A)  do { mpz_set(x->a,A->a);  mpz_set(x->b,A->b); } while (0)
#define iqc_equal(x,y)   (!mpz_cmp(x->a,y->a)&&!mpz_cmp(x->b,y->b))

#define IQC_CONSTRUCTOR(name) static void iqc_##name##_constructor(void) __attribute__ ((constructor)); \
                              static void iqc_##name##_constructor(void)

#include "serialization.h"

#endif
