/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include "elgamal.h"

#include "oper.h"
#include "random.h"
#include "gen_disc.h"

//Contructor
static mpz_t u; static iqc_t cau;
IQC_CONSTRUCTOR(elgamal) { mpz_init(u); iqc_init(cau); }

//Methods
void elgamal_gen(int bits, elgamal_pubk_t pub, elgamal_privk_t priv) {
  iqc_gen_disc2(pub->D, bits);
  iqc_set_disc(pub->D);

  iqc_random(pub->c);
  mpz_urandomb(priv->a, iqc_randst, bits>>1); mpz_add_ui(priv->a, priv->a, 2);
  iqc_exp(pub->a, pub->c, priv->a);
}

void elgamal_enc(elgamal_pubk_t pub, unsigned char M[ELGAMAL_ENC_LEN], elgamal_enc_t enc) {
  iqc_set_disc(pub->D);

  mpz_urandomb(u, iqc_randst, mpz_sizeinbase(pub->D,2)>>1); mpz_add_ui(u, u, 2);
  iqc_exp(enc->g, pub->c, u);

  iqc_exp(cau, pub->a, u);
  hash_state_t h;
  hash_init(h);
  hash_iqc(h, cau);
  hash_finish(h, enc->C);

  for (int i=0;i<ELGAMAL_ENC_LEN;i++) enc->C[i]^=M[i];
}

void elgamal_dec(elgamal_pubk_t pub,elgamal_privk_t priv,elgamal_enc_t enc,unsigned char M[ELGAMAL_ENC_LEN]) {
  iqc_set_disc(pub->D);

  iqc_exp(cau, enc->g, priv->a);
  hash_state_t h;
  hash_init(h);
  hash_iqc(h, cau);
  hash_finish(h, M);

  for (int i=0;i<ELGAMAL_ENC_LEN;i++) M[i]^=enc->C[i];
}

IMPLEMENT_INIT(elgamal_pubk) INIT_MPZ(D) INIT_IQC(c) INIT_IQC(a) IMPLEMENT_END
IMPLEMENT_CLER(elgamal_pubk) CLER_MPZ(D) CLER_IQC(c) CLER_IQC(a) IMPLEMENT_END
IMPLEMENT_LOAD(elgamal_pubk) LOAD_MPZ(D) LOAD_IQC(c) LOAD_IQC(a) IMPLEMENT_CLOSE
IMPLEMENT_SAVE(elgamal_pubk) SAVE_MPZ(D) SAVE_IQC(c) SAVE_IQC(a) IMPLEMENT_CLOSE

IMPLEMENT_INIT(elgamal_privk) INIT_MPZ(a) IMPLEMENT_END
IMPLEMENT_CLER(elgamal_privk) CLER_MPZ(a) IMPLEMENT_END
IMPLEMENT_LOAD(elgamal_privk) LOAD_MPZ(a) IMPLEMENT_CLOSE
IMPLEMENT_SAVE(elgamal_privk) SAVE_MPZ(a) IMPLEMENT_CLOSE

IMPLEMENT_INIT(elgamal_enc) INIT_IQC(g) INIT_DIG(C) IMPLEMENT_END
IMPLEMENT_CLER(elgamal_enc) CLER_IQC(g) CLER_DIG(C) IMPLEMENT_END
IMPLEMENT_LOAD(elgamal_enc) LOAD_IQC(g) LOAD_DIG(C) IMPLEMENT_CLOSE
IMPLEMENT_SAVE(elgamal_enc) SAVE_IQC(g) SAVE_DIG(C) IMPLEMENT_CLOSE

