/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include "gen_disc.h"

//Constructors
static mpz_t gen_disc, gen_p, gen_q, gen_b;
IQC_CONSTRUCTOR(gen_disc) { mpz_init(gen_disc); mpz_init(gen_p); mpz_init(gen_q); mpz_init(gen_b); }

//Methods
void iqc_gen_disc1(mpz_t d, unsigned long bits) {
  mpz_set_ui(gen_b, 2);
  mpz_mul_2exp(gen_b, gen_b, bits);
  do {
    mpz_urandomb(gen_p, iqc_randst, bits);
    mpz_setbit(gen_p,0); mpz_setbit(gen_p,1);
    mpz_add(gen_p, gen_p, gen_b);
  } while (!mpz_probab_prime_p(gen_p, 20));
  mpz_neg(d,gen_p);
}

void iqc_gen_disc2(mpz_t d, unsigned long bits) {
  bits=(bits+1)>>1;
  mpz_set_ui(gen_b, 2);
  mpz_mul_2exp(gen_b, gen_b, bits);
  do {
    mpz_urandomb(gen_p, iqc_randst, bits);
    mpz_setbit(gen_p,0);
    mpz_add(gen_p, gen_p, gen_b);
  } while (!mpz_probab_prime_p(gen_p, 20));
  do {
    mpz_urandomb(gen_q, iqc_randst, bits);
    mpz_setbit(gen_q,0);
    mpz_add(gen_q, gen_q, gen_b);
  } while (!(mpz_tstbit(gen_p,1)^mpz_tstbit(gen_q,1)) || !mpz_probab_prime_p(gen_q, 20));
  mpz_mul(d, gen_p, gen_q);
  mpz_neg(d, d);
}

void iqc_set_gen_disc1(unsigned long bits) {
  iqc_gen_disc1(gen_disc, bits);
  iqc_set_disc(gen_disc);
}

void iqc_set_gen_disc2(unsigned long bits) {
  iqc_gen_disc2(gen_disc, bits);
  iqc_set_disc(gen_disc);
}
