/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include "gps.h"

#include "oper.h"
#include "random.h"
#include "gen_disc.h"
#include "bits.h"

//Contructor
static mpz_t hnum; static iqc_t b,w;
IQC_CONSTRUCTOR(elgamal) { mpz_init(hnum); iqc_init(b); iqc_init(w); }

//Methods
void gps_gen(int bits, gps_pubk_t pub, gps_privk_t priv) {
  iqc_gen_disc2(pub->D, bits);
  iqc_set_disc(pub->D);

  iqc_random(pub->c);
  mpz_urandomb(priv->a, iqc_randst, 8*HASH_DIGEST_LEN); mpz_add_ui(priv->a, priv->a, 2);
  iqc_exp(pub->a, pub->c, priv->a);
}

void gps_sig(gps_pubk_t pub, gps_privk_t priv, hash_state_t doc, gps_sig_t sig) {
  iqc_set_disc(pub->D);

  mpz_urandomb(sig->s, iqc_randst, 8*2*HASH_DIGEST_LEN+80);
  iqc_exp(sig->r, pub->c, sig->s);

  unsigned char h[HASH_DIGEST_LEN];
  hash_iqc(doc, sig->r);
  hash_finish(doc, h);
  mpz_from_bytes(hnum, h, HASH_DIGEST_LEN);
  mpz_submul(sig->s, priv->a, hnum);
}

int gps_ver(gps_pubk_t pub, hash_state_t doc, gps_sig_t sig) {
  iqc_set_disc(pub->D);

  if (!iqc_valid_form(sig->r)) return 0;
  if (mpz_sgn(sig->s)>0 && mpz_sizeinbase(sig->s,2) > 8*2*HASH_DIGEST_LEN+80+1) return 0;
  if (mpz_sgn(sig->s)<0 && mpz_sizeinbase(sig->s,2) > 8*2*HASH_DIGEST_LEN+80) return 0;

  unsigned char h[HASH_DIGEST_LEN];
  hash_iqc(doc, sig->r);
  hash_finish(doc, h);
  mpz_from_bytes(hnum, h, HASH_DIGEST_LEN);

  iqc_exp(b, pub->a, hnum);
  iqc_exp(w, pub->c, sig->s);
  iqc_mul(b, b, w);

  return iqc_equal(b, sig->r);
}

IMPLEMENT_INIT(gps_pubk) INIT_MPZ(D) INIT_IQC(c) INIT_IQC(a) IMPLEMENT_END
IMPLEMENT_CLER(gps_pubk) CLER_MPZ(D) CLER_IQC(c) CLER_IQC(a) IMPLEMENT_END
IMPLEMENT_LOAD(gps_pubk) LOAD_MPZ(D) LOAD_IQC(c) LOAD_IQC(a) IMPLEMENT_CLOSE
IMPLEMENT_SAVE(gps_pubk) SAVE_MPZ(D) SAVE_IQC(c) SAVE_IQC(a) IMPLEMENT_CLOSE

IMPLEMENT_INIT(gps_privk) INIT_MPZ(a) IMPLEMENT_END
IMPLEMENT_CLER(gps_privk) CLER_MPZ(a) IMPLEMENT_END
IMPLEMENT_LOAD(gps_privk) LOAD_MPZ(a) IMPLEMENT_CLOSE
IMPLEMENT_SAVE(gps_privk) SAVE_MPZ(a) IMPLEMENT_CLOSE

IMPLEMENT_INIT(gps_sig) INIT_MPZ(s) INIT_IQC(r) IMPLEMENT_END
IMPLEMENT_CLER(gps_sig) CLER_MPZ(s) CLER_IQC(r) IMPLEMENT_END
IMPLEMENT_LOAD(gps_sig) LOAD_MPZ(s) LOAD_IQC(r) IMPLEMENT_CLOSE
IMPLEMENT_SAVE(gps_sig) SAVE_MPZ(s) SAVE_IQC(r) IMPLEMENT_CLOSE

