/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include "gq.h"

#include "oper.h"
#include "random.h"
#include "gen_disc.h"
#include "bits.h"

//Contructor
static mpz_t snum; static iqc_t b,w;
IQC_CONSTRUCTOR(elgamal) { mpz_init(snum); iqc_init(b); iqc_init(w); }

//Methods
void gq_gen(int bits, gq_pubk_t pub, gq_privk_t priv) {
  iqc_gen_disc2(pub->D, bits);
  iqc_set_disc(pub->D);

  iqc_random(priv->a);
  mpz_urandomb(pub->q, iqc_randst, 8*HASH_DIGEST_LEN); mpz_add_ui(pub->q, pub->q, 2);
  iqc_exp(pub->a, priv->a, pub->q);  
  iqc_invert(pub->a, pub->a);
}

void gq_sig(gq_pubk_t pub, gq_privk_t priv, hash_state_t doc, gq_sig_t sig) {
  iqc_set_disc(pub->D);

  iqc_random(b);
  iqc_exp(w, b, pub->q);
  hash_iqc(doc, w);
  hash_finish(doc, sig->s);

  mpz_from_bytes(snum, sig->s, HASH_DIGEST_LEN);
  iqc_exp(sig->w, priv->a, snum);
  iqc_mul(sig->w, sig->w, b);
}

int gq_ver(gq_pubk_t pub, hash_state_t doc, gq_sig_t sig) {
  iqc_set_disc(pub->D);

  if (!iqc_valid_form(sig->w)) return 0;

  iqc_exp(w, sig->w, pub->q);
  mpz_from_bytes(snum, sig->s, HASH_DIGEST_LEN);
  iqc_exp(b, pub->a, snum);
  iqc_mul(b, b, w);

  unsigned char h[HASH_DIGEST_LEN];
  hash_iqc(doc, b);
  hash_finish(doc, h);

  return memcmp(h, sig->s, HASH_DIGEST_LEN)==0;
}

IMPLEMENT_INIT(gq_pubk) INIT_MPZ(D) INIT_MPZ(q) INIT_IQC(a) IMPLEMENT_END
IMPLEMENT_CLER(gq_pubk) CLER_MPZ(D) CLER_MPZ(q) CLER_IQC(a) IMPLEMENT_END
IMPLEMENT_LOAD(gq_pubk) LOAD_MPZ(D) LOAD_MPZ(q) LOAD_IQC(a) IMPLEMENT_CLOSE
IMPLEMENT_SAVE(gq_pubk) SAVE_MPZ(D) SAVE_MPZ(q) SAVE_IQC(a) IMPLEMENT_CLOSE

IMPLEMENT_INIT(gq_privk) INIT_IQC(a) IMPLEMENT_END
IMPLEMENT_CLER(gq_privk) CLER_IQC(a) IMPLEMENT_END
IMPLEMENT_LOAD(gq_privk) LOAD_IQC(a) IMPLEMENT_CLOSE
IMPLEMENT_SAVE(gq_privk) SAVE_IQC(a) IMPLEMENT_CLOSE

IMPLEMENT_INIT(gq_sig) INIT_IQC(w) INIT_DIG(s) IMPLEMENT_END
IMPLEMENT_CLER(gq_sig) CLER_IQC(w) CLER_DIG(s) IMPLEMENT_END
IMPLEMENT_LOAD(gq_sig) LOAD_IQC(w) LOAD_DIG(s) IMPLEMENT_CLOSE
IMPLEMENT_SAVE(gq_sig) SAVE_IQC(w) SAVE_DIG(s) IMPLEMENT_CLOSE

