/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include "mpz_sqrtmod.h"

//Constructor
static mpz_t p4, b, t, y;
IQC_CONSTRUCTOR(mpz_sqrtmod) { mpz_init(p4); mpz_init(b); mpz_init(t); mpz_init(y); }

  //Methods
int mpz_sqrtmod(mpz_t r, mpz_t a, mpz_t p) {
  if (mpz_cmp_ui(p,2)==0) {mpz_set_ui(r,mpz_tstbit(a,0)); return mpz_tstbit(a,0);}
  if (mpz_legendre(a, p)==-1) return -1;
  if (mpz_divisible_p(a,p)) { mpz_set_ui(r,0); return 0; }
  
  //p congruent to 3 mod 4
  if (mpz_tstbit(p,1)==1) { 
    mpz_add_ui(p4,p,1); 
    mpz_tdiv_q_2exp(p4,p4,2); 
    mpz_powm(r, a, p4, p); 
    return 1; 
  }

  //p congruent to 5 mod 8
  if (mpz_tstbit(p,2)==1) {
    mpz_sub_ui(p4,p,1);
    mpz_tdiv_q_2exp(p4,p4,2); 
    mpz_powm(r, a, p4, p); 
    if (mpz_cmp_ui(r,1)==0) {
      mpz_add_ui(p4, p4, 1);
      mpz_tdiv_q_2exp(p4,p4,1);
      mpz_powm(r, a, p4, p);
    } else {
      mpz_sub_ui(p4, p4, 1);
      mpz_tdiv_q_2exp(p4,p4,1);
      mpz_mul_2exp(r, a, 2);
      mpz_powm(r, r, p4, p);
      mpz_mul(r, r, a);
      mpz_mul_2exp(r, r, 1);
      mpz_mod(r, r, p);
    }
    return 1;
  }
  
  //ok, p congruent to 1 mod 8, Shans-Tonelli kicks in
  mpz_set_ui(y,2);
  while(mpz_legendre(y, p) != -1) mpz_add_ui(y, y, 1);
 
  mpz_sub_ui(r, p, 1);  unsigned long rlen = mpz_scan1(r, 0), m;  mpz_tdiv_q_2exp(r,r,rlen); 
  mpz_powm(y, y, r, p);   

  mpz_tdiv_q_2exp(r,r,1);  mpz_powm(b, a, r, p); 
 
  mpz_mul(r, a, b);  mpz_mod(r, r, p);  
  mpz_mul(b, r, b);  mpz_mod(b, b, p);   

  while(mpz_cmp_ui(b, 1)) {    
    mpz_mul(t, b, b);  mpz_mod(t, t, p);
    for(m = 1; mpz_cmp_ui(t, 1); m++) { mpz_mul(t, t, t); mpz_mod(t, t, p); }   
 
    mpz_set_ui(t, 0);
    mpz_setbit(t, rlen - m - 1);
    mpz_powm(t, y, t, p);     
    mpz_mul(y, t, t);    
 
    rlen = m;     
    mpz_mul(r, r, t); mpz_mod(r, r, p);       
    mpz_mul(b, b, y); mpz_mod(b, b, p);
  }  
  return 1;
}
