/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include "oper_exp.h"
#include "oper.h"

//Constructors
static mpz_t exp_d; static iqc_t exp_z, exp_zinv;
IQC_CONSTRUCTOR(oper_exp) { mpz_init(exp_d); iqc_init(exp_z); iqc_init(exp_zinv); }

//Methods
void iqc_exp_basic(iqc_t x, iqc_t a, mpz_t e) {
  if (mpz_sgn(e)==0) {iqc_identity(x); return; }
  if (mpz_sgn(e)<0) {iqc_invert(exp_z,a); mpz_neg(exp_d,e);} else {iqc_set(exp_z,a); mpz_set(exp_d,e); }
  if (mpz_tstbit(exp_d,0)) iqc_set(x,exp_z); else iqc_identity(x);

  int dlen=mpz_sizeinbase(exp_d, 2);
  for (int i=1;i<dlen;i++) {
    iqc_sqr(exp_z,exp_z);
    if (mpz_tstbit(exp_d,i)) iqc_mul(x,x,exp_z);
  }
}

void iqc_exp_signed(iqc_t x, iqc_t a, mpz_t e) {
  if (mpz_sgn(e)==0) {iqc_identity(x); return; }
  if (mpz_sgn(e)<0) {iqc_invert(exp_z,a); mpz_neg(exp_d,e);} else {iqc_set(exp_z,a); mpz_set(exp_d,e); }

  iqc_identity(x);
  int dlen=mpz_sizeinbase(exp_d, 2);
  for (int i=0,c=0;i<dlen || c;i++) {
    int b=c+mpz_tstbit(exp_d,i); c=(b+mpz_tstbit(exp_d,i+1))>>1; b-=c<<1;
    if (b==1) iqc_mul(x,x,exp_z);
    if (b==-1) { iqc_invert(exp_zinv,exp_z); iqc_mul(x,x,exp_zinv); }
    if (i+1<dlen || c) iqc_sqr(exp_z,exp_z);
  }
}
