/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include "oper_nucomp.h"
#include "oper_basic.h"

//Constructor
static mpz_t L, part_q, part_r;
static mpz_t nu_d, nu_dd, nu_d1, nu_g, nu_e, nu_f, nu_c, nu_c1, nu_c2, nu_a1, nu_a2, nu_a3, nu_b3, nu_c3;
static mpz_t nu_A, nu_B, nu_C, nu_s, nu_n, nu_u, nu_v, nu_vv, nu_Q1, nu_Q2, nu_Q3, nu_Q4;
IQC_CONSTRUCTOR(oper_nucomp) { 
  mpz_init(L);     mpz_init(part_q); mpz_init(part_r); 
  mpz_init(nu_d);  mpz_init(nu_dd);  mpz_init(nu_d1); mpz_init(nu_g);   mpz_init(nu_e);  mpz_init(nu_f);
  mpz_init(nu_c);  mpz_init(nu_c1);  mpz_init(nu_c2); mpz_init(nu_a1);  mpz_init(nu_a2);  
  mpz_init(nu_a3); mpz_init(nu_b3);  mpz_init(nu_c3); 
  mpz_init(nu_A);  mpz_init(nu_B);   mpz_init(nu_C);
  mpz_init(nu_s);  mpz_init(nu_n);   mpz_init(nu_u);  mpz_init(nu_v);   mpz_init(nu_vv);
  mpz_init(nu_Q1); mpz_init(nu_Q2);  mpz_init(nu_Q3); mpz_init(nu_Q4); 
}

//Helper methods
static void parteucl(mpz_t d, mpz_t v, mpz_t dd, mpz_t vv, mpz_t a, mpz_t b) {
  mpz_set(dd, a); mpz_set_ui(vv, 0); mpz_set(d, b); mpz_set_ui(v, 1); int z=0;
  while (mpz_cmpabs(d,L)>0) {
    mpz_fdiv_qr(part_q,part_r,dd,d);
    mpz_submul(vv,part_q,v); mpz_swap(vv,v);
    mpz_set(dd,d); mpz_set(d,part_r);
    z++;
  }
  if (z&1) { mpz_neg(d,d); mpz_neg(v,v); }
}


//Methods
void iqc_nucomp_init(void) {
  mpz_neg(L, iqc_disc);
  mpz_root(L, L, 4);
}

void iqc_mul_nucomp(iqc_t x, iqc_t a, iqc_t b) {
#ifdef DEBUG
  assert(iqc_valid_form(a));
  assert(iqc_valid_form(b));
#endif

  if (mpz_cmp(a->a, b->a)<0) { struct iqc_struct *t=a; a=b; b=t; }
  mpz_neg(nu_c1, iqc_disc);            mpz_addmul(nu_c1, a->b, a->b);
  mpz_divexact(nu_c1, nu_c1, a->a);    mpz_tdiv_q_2exp(nu_c1, nu_c1, 2);
  mpz_neg(nu_c2, iqc_disc);            mpz_addmul(nu_c2, b->b, b->b);
  mpz_divexact(nu_c2, nu_c2, b->a);    mpz_tdiv_q_2exp(nu_c2, nu_c2, 2);

  mpz_add(nu_s, a->b, b->b);  mpz_tdiv_q_2exp(nu_s, nu_s, 1);  mpz_sub(nu_n, b->b, nu_s);
  mpz_gcdext(nu_d, nu_v, nu_u, a->a, b->a);
  
  if (mpz_cmp_ui(nu_d,1)==0) { 
    mpz_mul(nu_A, nu_u, nu_n); mpz_neg(nu_A, nu_A); mpz_set(nu_d1, nu_d); 
    mpz_set(nu_a1, a->a); mpz_set(nu_a2, b->a);
  }
  else if (mpz_divisible_p(nu_s, nu_d)) {
    mpz_mul(nu_A, nu_u, nu_n); mpz_neg(nu_A, nu_A); mpz_set(nu_d1, nu_d); 
    mpz_divexact(nu_a1, a->a, nu_d); mpz_divexact(nu_a2, b->a, nu_d); mpz_divexact(nu_s, nu_s, nu_d);    
  } else {
    mpz_gcdext(nu_d1, nu_Q1, nu_Q2, nu_s, nu_d); //we dont need Q2
    if (mpz_cmp_ui(nu_d1,1)>0) {
      mpz_divexact(nu_a1, a->a, nu_d1); mpz_divexact(nu_a2, b->a, nu_d1);
      mpz_divexact(nu_s, nu_s, nu_d1);  mpz_divexact(nu_d, nu_d, nu_d1);
    } else { mpz_set(nu_a1, a->a);      mpz_set(nu_a2, b->a); }
    mpz_mod(nu_c, nu_c1, nu_d);
    mpz_mod(nu_g, nu_c2, nu_d); mpz_mul(nu_g, nu_g, nu_v); mpz_addmul(nu_g, nu_u, nu_c);
    mpz_neg(nu_g, nu_g);        mpz_mul(nu_g, nu_g, nu_Q1);mpz_mod(nu_g, nu_g, nu_d);

    mpz_divexact(nu_A, nu_a1, nu_d); mpz_mul(nu_A, nu_A, nu_g);
    mpz_divexact(nu_g, nu_n, nu_d);  mpz_submul(nu_A, nu_u, nu_g);
  }
  mpz_mod(nu_A, nu_A, nu_a1);
  mpz_mul_2exp(nu_a3, nu_A, 1);  if (mpz_cmp(nu_a1, nu_a3)<0) mpz_sub(nu_A, nu_A, nu_a1);

  parteucl(nu_d, nu_v, nu_dd, nu_vv, nu_a1, nu_A);
  if (mpz_sgn(nu_vv)==0) {
    mpz_mul(nu_Q1, nu_a2, nu_d);   mpz_add(nu_Q2, nu_Q1, nu_n);   mpz_divexact(nu_f, nu_Q2, nu_dd);
    mpz_set(nu_g, nu_c2);          mpz_addmul(nu_g, nu_d, nu_s);  mpz_divexact(nu_g, nu_g, nu_dd);
    mpz_mul(nu_a3, nu_a2, nu_dd);
    mpz_mul_2exp(nu_b3, nu_Q1, 1); mpz_add(nu_b3, nu_b3, b->b);
    mpz_mul(nu_c3, nu_d, nu_f);   mpz_addmul(nu_c3, nu_g, nu_d1);
    iqc_reduce_lowlevel(x, nu_a3, nu_b3, nu_c3);
    return;
  }
  mpz_mul(nu_b3, nu_n, nu_vv);   mpz_addmul(nu_b3, nu_a2, nu_dd);  mpz_divexact(nu_b3, nu_b3, nu_a1);
  mpz_mul(nu_Q1, nu_b3, nu_d);   mpz_add(nu_Q2, nu_Q1, nu_n);      mpz_divexact(nu_f, nu_Q2, nu_dd);
  mpz_mul(nu_e, nu_s, nu_dd);    mpz_addmul(nu_e, nu_c2, nu_vv);   mpz_divexact(nu_e, nu_e, nu_a1);
  mpz_mul(nu_Q3, nu_e, nu_v);    mpz_sub(nu_Q4, nu_Q3, nu_s);      mpz_divexact(nu_g, nu_Q4, nu_vv);
  if (mpz_cmp_ui(nu_d1, 1)>0) { mpz_mul(nu_v, nu_v, nu_d1);  mpz_mul(nu_vv, nu_vv, nu_d1); }
  mpz_mul(nu_a3, nu_dd, nu_b3);  mpz_addmul(nu_a3, nu_vv, nu_e);
  mpz_add(nu_Q4, nu_Q4, nu_Q3);  mpz_add(nu_b3, nu_Q1, nu_Q2);     mpz_addmul(nu_b3, nu_Q4, nu_d1);
  mpz_mul(nu_c3, nu_d, nu_f);    mpz_addmul(nu_c3, nu_v, nu_g);
  iqc_reduce_lowlevel(x, nu_a3, nu_b3, nu_c3);
}

void iqc_sqr_nucomp(iqc_t x, iqc_t a) {
#ifdef DEBUG
  assert(iqc_valid_form(a));
#endif
  mpz_neg(nu_c, iqc_disc);            mpz_addmul(nu_c, a->b, a->b);
  mpz_divexact(nu_c, nu_c, a->a);     mpz_tdiv_q_2exp(nu_c, nu_c, 2);
  
  mpz_gcdext(nu_d1,nu_vv,nu_v,a->a,a->b);
  mpz_divexact(nu_A,a->a,nu_d1); mpz_divexact(nu_B,a->b,nu_d1);
  mpz_mul(nu_C,nu_c,nu_v);       mpz_neg(nu_C,nu_C);     mpz_mod(nu_C,nu_C,nu_A);
  mpz_mul_2exp(nu_c3,nu_C,1);    if (mpz_cmp(nu_A,nu_c3)<0) mpz_sub(nu_C,nu_C,nu_A);
  parteucl(nu_d,nu_v,nu_dd,nu_vv,nu_A,nu_C);

  if (mpz_sgn(nu_vv)==0) {
    mpz_set(nu_g, nu_c);    mpz_addmul(nu_g,nu_B,nu_d);   mpz_divexact(nu_g,nu_g,nu_dd);
    mpz_mul(nu_a3, nu_dd, nu_dd);    mpz_mul(nu_c3, nu_d, nu_d);
    mpz_add(nu_b3, nu_d, nu_dd);     mpz_mul(nu_b3, nu_b3, nu_b3);
    mpz_add(nu_b3, nu_b3, a->b);     mpz_sub(nu_b3, nu_b3, nu_a3);  mpz_sub(nu_b3, nu_b3, nu_c3);
    mpz_addmul(nu_c3, nu_g, nu_d1);
    iqc_reduce_lowlevel(x, nu_a3, nu_b3, nu_c3);
    return;
  }
  
  mpz_mul(nu_e, nu_c, nu_vv);   mpz_addmul(nu_e, nu_B, nu_dd);   mpz_divexact(nu_e, nu_e, nu_A);
  mpz_mul(nu_b3, nu_e, nu_v);   mpz_sub(nu_g, nu_b3, nu_B);      mpz_divexact(nu_g, nu_g, nu_vv);
  mpz_addmul(nu_b3, nu_vv, nu_g);
  if (mpz_cmp_ui(nu_d1,1)>0) { 
    mpz_mul(nu_b3, nu_b3, nu_d1); mpz_mul(nu_v,nu_v,nu_d1); mpz_mul(nu_vv,nu_vv,nu_d1); 
  }
  mpz_mul(nu_a3, nu_dd, nu_dd); mpz_mul(nu_c3, nu_d, nu_d);
  mpz_add(x->b, nu_d, nu_dd); mpz_addmul(nu_b3, x->b, x->b);
  mpz_sub(nu_b3, nu_b3, nu_a3); mpz_sub(nu_b3, nu_b3, nu_c3);
  mpz_addmul(nu_a3, nu_e, nu_vv); mpz_addmul(nu_c3, nu_g, nu_v);
  iqc_reduce_lowlevel(x, nu_a3, nu_b3, nu_c3);
}
