/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include "random.h"
#include "oper.h"
#include "mpz_sqrtmod.h"

//Constructor
static iqc_t random_a; static mpz_t random_p;
IQC_CONSTRUCTOR(random) { iqc_init(random_a); mpz_init(random_p); }

//Helper methods
static int iqc_is_2_prime(void) { 
  return !(mpz_tstbit(iqc_disc,0) & mpz_tstbit(iqc_disc,2));  //D%8 is not 5
}

static int iqc_prime_ui(iqc_t x, unsigned long p) {
  if (p==2) { mpz_set_ui(x->a,2); mpz_set_ui(x->b,mpz_tstbit(iqc_disc,0)); return iqc_is_2_prime(); }
  mpz_set_ui(x->a, p);
  if (mpz_sqrtmod(x->b,iqc_disc,x->a)<=0) return 0;
  if (mpz_tstbit(iqc_disc, 0) != mpz_tstbit(x->b,0)) mpz_sub(x->b,x->a,x->b);
  iqc_reduce(x, x);
  return 1;
}

//Methods
void iqc_random(iqc_t x) {
  iqc_identity(x);
  int num=mpz_size(iqc_disc);
  for (int i=0;i<num+(num>>3);i++) {
    unsigned long p=1+2*gmp_urandomb_ui(iqc_randst, sizeof(unsigned long)*4-1);
    if (p==1) p=2;

    mpz_set_ui(random_p, p);
    if (!mpz_probab_prime_p(random_p, 20) || mpz_kronecker_ui(iqc_disc, p)<0) {i--; continue; }
    if (!iqc_prime_ui(random_a, p)) { i--; continue; } //should never happen
    if (gmp_urandomb_ui(iqc_randst,1)) iqc_invert(random_a, random_a);
    iqc_mul(x, x, random_a);
  }
}
