/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#ifndef LIBIQC_SERIALIZATION_MILAN_STRAKA_2008
#define LIBIQC_SERIALIZATION_MILAN_STRAKA_2008

#define SERIALIZABLE_INIT(name)  void name##_init(name##_t what)
#define SERIALIZABLE_CLER(name)  void name##_clear(name##_t what)
#define SERIALIZABLE_LOAD(name)  void name##_load(name##_t what, const char* fname)
#define SERIALIZABLE_SAVE(name)  void name##_save(name##_t what, const char* fname)
#define SERIALIZABLE(name)       SERIALIZABLE_INIT(name); SERIALIZABLE_CLER(name); \
                                 SERIALIZABLE_LOAD(name); SERIALIZABLE_SAVE(name)
#define IMPLEMENT_INIT(name)			\
  SERIALIZABLE_INIT(name) {
#define IMPLEMENT_CLER(name)			\
  SERIALIZABLE_CLER(name) {
#define IMPLEMENT_END }
#define IMPLEMENT_LOAD(name)				\
  SERIALIZABLE_LOAD(name) {				\
    FILE *f;						\
    if (!(f=fopen(fname,"rt"))) {			\
      fprintf(stderr, "Cannot open file %s!\n", fname); \
      exit(1);						\
    }							\
    fscanf(f, "Structure " #name "\n");
#define IMPLEMENT_SAVE(name)				\
  SERIALIZABLE_SAVE(name) {				\
    FILE *f;						\
    if (!(f=fopen(fname,"wt"))) {			\
      fprintf(stderr, "Cannot open file %s!\n", fname); \
      exit(1);						\
    }							\
    fprintf(f, "Structure " #name "\n");
#define IMPLEMENT_CLOSE				\
  fclose(f);					\
  }

#define INIT_MPZ(name) mpz_init(what->name);
#define CLER_MPZ(name) mpz_clear(what->name);
#define SAVE_MPZ(name) gmp_fprintf(f, #name ": %Zd\n", what->name);
#define LOAD_MPZ(name) if (gmp_fscanf(f, #name ": %Zd\n", what->name)!=1) \
    { fprintf(stderr, "Cannot load member " #name "!\n"); exit(1); }

#define INIT_IQC(name) iqc_init(what->name);
#define CLER_IQC(name) iqc_clear(what->name);
#define SAVE_IQC(name) gmp_fprintf(f, #name ": (%Zd,%Zd)\n", what->name->a, what->name->b);
#define LOAD_IQC(name) if (gmp_fscanf(f, #name ": (%Zd,%Zd)\n", what->name->a, what->name->b)!=2) \
    { fprintf(stderr, "Cannot load member " #name "!\n"); exit(1); }

#define INIT_DIG(name)
#define CLER_DIG(name)
#define SAVE_DIG(name) fputs(#name ":", f); for (int i=0;i<HASH_DIGEST_LEN;i++)	\
			  fprintf(f, " %02X",what->name[i]); fputc('\n', f);
#define LOAD_DIG(name) fscanf(f,#name ":"); for (int i=0,x;i<HASH_DIGEST_LEN;what->name[i++]=x)	\
			  if (fscanf(f,"%X",&x)!=1)	\
			      { fprintf(stderr, "Cannot load member " #name "!\n"); exit(1); }

#endif
