/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include "iqc.h"
#include "timing.h"

int main(void) {
  int b;
  scanf("%d",&b);
  iqc_set_gen_disc2(b);

  mpz_t e; mpz_init(e);
  iqc_t x,y,a;
  iqc_init(x); iqc_init(y); iqc_init(a);

  gmp_randstate_t r;
  gmp_randinit_default(r);
  
  while(1) {
    iqc_random(a);
    mpz_urandomb(e, r, b>>1);

    STOPWATCH_START();
    iqc_exp_basic(x, a, e);
    STOPWATCH_STOP("Basic exp");

    STOPWATCH_START();
    iqc_exp_signed(y, a, e);
    STOPWATCH_STOP("Signed exp");

    if (!iqc_equal(x,y)) {
      gmp_printf("(%Zd,%Zd), (%Zd, %Zd)\n", x->a, x->b, y->a, y->b);
      puts("ERROR"); 
      return 1;
    }
  }
  return 0;
}
