/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include "iqc.h"
#include "timing.h"

int main(void) {
  int bits;
  scanf("%d",&bits); bits=(bits+1)>>1;
#define Z(a) mpz_t a; mpz_init(a)
  Z(p); Z(p1); Z(q); Z(q1); Z(pinvq); Z(n);
  Z(x); Z(y); Z(a); Z(e); Z(ap); Z(ep); Z(aq); Z(eq); Z(xp); Z(xq);
  gmp_randstate_t r;
  gmp_randinit_default(r);  

  mpz_urandomb(p, r, bits);
  mpz_setbit(p, bits);
  mpz_nextprime(p, p);

  mpz_urandomb(q, r, bits);
  mpz_setbit(q, bits);
  mpz_nextprime(q, q);

  mpz_mul(n, p, q);
  mpz_sub_ui(p1,p,1);
  mpz_sub_ui(q1,q,1);  
  mpz_invert(pinvq, p, q);
    
  while(1) {    
    mpz_urandomm(a, r, n);
    mpz_urandomm(e, r, n);

    STOPWATCH_START();
    for (int i=0;i<5;i++) {
	mpz_mod(ap, a, p); mpz_mod(ep, e, p1); mpz_powm(xp, ap, ep, p);
	mpz_mod(aq, a, q); mpz_mod(eq, e, q1); mpz_powm(xq, aq, eq, q);
	mpz_sub(x, xq, xp); mpz_mul(x, x, pinvq); mpz_mod(x, x, q); 
	mpz_mul(x, x, p); mpz_add(x, x, xp);
    }
    STOPWATCH_STOP("RSA signature");
    mpz_powm(y, a, e, n);
    if (mpz_cmp(x,y)!=0) {fprintf(stderr, "ERROR!\n"); return 1;}

    STOPWATCH_START();
    for (int i=0;i<5;i++) mpz_powm_ui(x,a,0x10001,n);
    STOPWATCH_STOP("RSA verification");
  }
  return 0;
}
