/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include "iqc.h"
#include "timing.h"

int main(int argc, char *argv[]) {
  if (argc<=4) {
    fprintf(stderr,"Usage: elgamal_dec pubk_file privk_file file_to_decrypt decrypted_filename!\n");
    return 1;
  }

  elgamal_pubk_t pub;              elgamal_privk_t priv;              elgamal_enc_t enc;
  elgamal_pubk_init(pub);          elgamal_privk_init(priv);          elgamal_enc_init(enc);
  elgamal_pubk_load(pub, argv[1]); elgamal_privk_load(priv, argv[2]); elgamal_enc_load(enc, argv[3]);
  unsigned char dec[ELGAMAL_ENC_LEN];

  STOPWATCH_START();
  elgamal_dec(pub, priv, enc, dec);
  STOPWATCH_STOP("The decryption took");

  FILE *f=fopen(argv[4], "wb");
  if (!f) {
    fprintf(stderr, "Cannot open file %s!\n", argv[4]);
    return 1;
  }
  if (fwrite(dec, 1, ELGAMAL_ENC_LEN, f)!=ELGAMAL_ENC_LEN) {
    fprintf(stderr, "Cannot write %d bytes to file %s!\n", ELGAMAL_ENC_LEN, argv[4]);
    return 1;
  }
  fclose(f);

  elgamal_pubk_clear(pub);         elgamal_privk_clear(priv);         elgamal_enc_clear(enc);

  return 0;
}
