/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include "iqc.h"
#include "timing.h"

int main(int argc, char *argv[]) {
  if (argc<=3) {
    fprintf(stderr, "Usage: elgamal_enc pubk_file file_to_encrypt encrypted_filename!\n");
    return 1;
  }

  elgamal_pubk_t pub;               elgamal_enc_t enc;
  elgamal_pubk_init(pub);           elgamal_enc_init(enc);
  elgamal_pubk_load(pub, argv[1]);

  unsigned char toenc[ELGAMAL_ENC_LEN];
  FILE *f=fopen(argv[2], "rb");
  if (!f) {
    fprintf(stderr, "Cannot open file %s!\n", argv[2]);
    return 1;
  }
  if (fread(toenc, 1, ELGAMAL_ENC_LEN, f)!=ELGAMAL_ENC_LEN) {
    fprintf(stderr, "The file %s does not contain %d bytes!\n", argv[2], ELGAMAL_ENC_LEN);
    return 1;

  }
  fclose(f);

  STOPWATCH_START();
  elgamal_enc(pub, toenc, enc);
  STOPWATCH_STOP("The encryption took");

  elgamal_enc_save(enc, argv[3]);
  elgamal_enc_clear(enc);          elgamal_pubk_clear(pub);

  return 0;
}
