/*
 * This file was written by Milan Straka as a part of iqc library.
 * Copyright (C) 2008 Milan Straka.
 * 
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this program.  If not, see 
 * <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include "iqc.h"
#include "timing.h"
#define BLOCK_SIZE 4096

int main(int argc, char *argv[]) {
  if (argc<=4) {
    fprintf(stderr, "Usage: gq_sig pubk_file privk_file message_file signature_file!\n");
    return 1;
  }

  gq_pubk_t pub;              gq_privk_t priv;              gq_sig_t sig;
  gq_pubk_init(pub);          gq_privk_init(priv);          gq_sig_init(sig);
  gq_pubk_load(pub, argv[1]); gq_privk_load(priv, argv[2]);

  unsigned char buffer[BLOCK_SIZE];
  FILE *f=fopen(argv[3], "rb");
  if (!f) {
    fprintf(stderr, "Cannot open file %s!\n", argv[3]);
    return 1;
  }
  hash_state_t h;
  hash_init(h);
  for (int len; (len=fread(buffer, 1, BLOCK_SIZE, f)); hash_data(h, buffer, len)) ;
  fclose(f);

  STOPWATCH_START();
  gq_sig(pub, priv, h, sig);
  STOPWATCH_STOP("The signature took");

  gq_sig_save(sig, argv[4]);
  gq_pubk_clear(pub);          gq_privk_clear(priv);        gq_sig_clear(sig);
 
  return 0;
}
